<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:cis="http://zlab.bu.edu/schema/cisml"
 xmlns:mem="http://meme.nbcr.net/schema/meme"
 xmlns:func="http://exslt.org/functions"
 xmlns:math="http://exslt.org/math"
 extension-element-prefixes="func"
>
  <xsl:output method="text" />

  <!-- A stylesheet to create a WIG formated file from CisML file.-->

  <xsl:template match="/*">
    <xsl:apply-templates select="cis:pattern">
      <xsl:sort order="ascending" data-type="text" select="@name"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="cis:pattern">
    <!-- Each motif goes into a separate track -->
    <xsl:text>track type=wiggle_0 name=&#34;motif </xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text>&#34; description=&#34;</xsl:text>
    <xsl:value-of select="../cis:program-name"/>
    <xsl:text> scan of motif </xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text>&#34; windowingFunction=maximum alwaysZero=on</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:apply-templates select="cis:scanned-sequence">
      <xsl:sort order="ascending" data-type="text" select="@name"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="cis:scanned-sequence">
  
    <xsl:if test="count(./cis:matched-element) &gt; 0">
      <xsl:text>variableStep </xsl:text>
      <xsl:text>chrom=</xsl:text>
      <xsl:value-of select="@name"/>
      <xsl:text> span=</xsl:text>
      <xsl:choose>
        <xsl:when test="cis:matched-element[1]/@stop &gt; cis:matched-element[1]/@start">
          <xsl:value-of select="cis:matched-element[1]/@stop - cis:matched-element[1]/@start + 1"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="cis:matched-element[1]/@start - cis:matched-element[1]/@stop + 1"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>&#10;</xsl:text>
      <xsl:apply-templates select="cis:matched-element">
        <xsl:sort order="ascending" data-type="number" select="mem:min(@start, @stop)"/>
      </xsl:apply-templates>
    </xsl:if>
  </xsl:template>

  <xsl:template match="cis:matched-element">
    <xsl:value-of select="mem:min(@start, @stop)"/>
    <xsl:text>&#9;</xsl:text>
    <xsl:value-of select="-(math:log(@pvalue) div math:log(2.0))"/>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <func:function name="mem:min">
    <xsl:param name="e1"/>
    <xsl:param name="e2"/>

    <xsl:variable name="x">
      <xsl:choose>
        <xsl:when test="number($e1) &gt; number($e2)">
          <xsl:value-of select="$e2"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$e1"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <func:result select="$x" />

  </func:function>

</xsl:stylesheet>
